﻿//Author: Fabrizio 'z4g0' Zagaglia
//License: (CC) creative commons 
Shader "Xilo"
{
	Properties
	{
		_MainTex("Base (RGB) Trans (A)", 2D) = "white" {}
		_LinesDensity("_LinesDensity",Range(50,400)) = 160
		_LineLen("_LineLen",Range(0,2)) = 0.4
		_LineDistortion("_LineDistortion",Range(0,400)) = 10
		_LineIntensityX("_LineIntensity H",Range(0,1)) = 1
		_LineIntensityY("_LineIntensity V", Range(0, 1)) = 1

		_LightWrap("_LightWrap",Range(0,4)) = 1
		_BorderSize("_BorderSize",Range(0,5)) = 3
	}

		SubShader
	{
		Tags{ "Queue" = "Transparent" "IgnoreProjector" = "True" "RenderType" = "Transparent" }
		ZWrite Off Lighting Off Cull Off Fog{ Mode Off } Blend SrcAlpha OneMinusSrcAlpha
		LOD 110

		Pass
	{
		CGPROGRAM
#pragma vertex vert
#pragma fragment frag 
#pragma fragmentoption ARB_precision_hint_fastest
#include "UnityCG.cginc"

		sampler2D _MainTex;
		float4 _MainTex_ST;
		float4 _MainTex_TexelSize;
		float  _LinesDensity;
		float  _LineLen;
		float  _LineDistortion;
		float  _LineIntensityX, _LineIntensityY;
		float  _LightWrap;
		float  _BorderSize;

		struct vertIN
		{
			float4 vertex : POSITION;
			float4 color : COLOR;
			float2 texcoord : TEXCOORD0;
			float2 texcoord1 : TEXCOORD1;
		};

		struct fragIN
		{
			float4 vertex : SV_POSITION;
			fixed4 color : COLOR;
			float2 texcoord : TEXCOORD0;
			float4 screenPos    : TEXCOORD1;
			float4 screenObjPos : TEXCOORD2;
		};

		fragIN vert(vertIN v)
		{
			fragIN o;
			o.vertex = mul(UNITY_MATRIX_MVP, v.vertex);
			o.color = v.color;
			o.texcoord = v.texcoord;
			o.screenPos = ComputeScreenPos(o.vertex);
			o.screenObjPos = ComputeScreenPos(mul(UNITY_MATRIX_MVP, float4(0, 0, 0, 1)));
			return o;
		}


		float2 Rotate(fixed2 v, float angle) {
			float ca = cos(angle);
			float sa = sin(angle);
			return float2(ca*v.x - sa*v.y, sa*v.x + ca*v.y);
		}

		fixed4 frag(fragIN i) : SV_Target
		{
			fixed3 col  = tex2D(_MainTex, i.texcoord);

			//border
			float borderSize = _MainTex_TexelSize.x*_BorderSize;
			fixed3 coldx = tex2D(_MainTex, i.texcoord + float2(borderSize,0));
			fixed3 colsx = tex2D(_MainTex, i.texcoord - float2(borderSize, 0));
			fixed3 colup = tex2D(_MainTex, i.texcoord + float2(0, borderSize));
			fixed3 coldn = tex2D(_MainTex, i.texcoord - float2(0, borderSize));
			float border = 1-abs(col - (coldx + colsx + colup + coldn) / 4);

			if (border < 0.9) border = 0; //soglia todo
		
			//col = normalize(col);
			fixed3 ldir = _WorldSpaceLightPos0.xyz;
			float  dl = dot(col, ldir);
			float3  cr = cross(normalize(col), ldir);
			float angle = atan2(ldir.x, ldir.y)+dl*0.2+i.texcoord.y*0.2;
			float2 rotatedUV = Rotate(i.texcoord - 0.5, angle) + 0.5;

			//light
			float  lit = dot(col, ldir);
			float  bn = lit;
			bn *= _LightWrap;
			//backlight
			bn += dot(col+0.5, ldir+0.45f);

			//pattern
			float phase = (rotatedUV.x*_LineIntensityX + rotatedUV.y*_LineIntensityY)*
						  (_LinesDensity + cos(i.texcoord.y*_LineDistortion*_LineIntensityY + i.texcoord.x*_LineDistortion*_LineIntensityX));
			
			bn -= abs(sin(phase))*_LineLen;

			bn += bn * bn;
			//add borders
			bn += 1.5- border;
			return float4(step(1-bn,0).rrr,1);

		}
		ENDCG
	}
	}
}
